@@ -153,9 +153,17 @@ def make_flags(prefix, debug):
     flags.append("--disable-shared")
     flags.append("--disable-examples")
 
-    # tls 1.3
-    flags.append("--enable-tls13")
-    flags.append("--enable-sslv3")
+    # dtls 1.3
+    flags.append("--enable-dtls13")
+
+    # dtls
+    flags.append("--enable-dtls")
+
+    # crl
+    flags.append("--enable-crl")
+
+    # openssl extra
+    flags.append("--enable-opensslextra")
 
     # for urllib3 - requires SNI (tlsx), options (openssl compat), peer cert
     flags.append("--enable-tlsx")
@@ -212,6 +220,7 @@ def build_wolfssl(ref, debug=False):
 
 
 def make_optional_func_list(libwolfssl_path, funcs):
+    sys.stderr.write("\nlibwolfssl Path: %s\n" % libwolfssl_path)
     if libwolfssl_path.endswith(".so"):
         libwolfssl = cdll.LoadLibrary(libwolfssl_path)
         defined = []
@@ -282,6 +291,13 @@ else:
         generate_libwolfssl()
         get_libwolfssl()
 
+# default values
+OLDTLS_ENABLED = 1
+
+if featureDetection:
+    OLDTLS_ENABLED = 0 if '#define NO_OLD_TLS' in optionsHeaderStr else 1
+
+sys.stderr.write("\nOLDTLS: %d\n" % OLDTLS_ENABLED)
 
 WolfFunction = namedtuple("WolfFunction", ["name", "native_sig", "ossl_sig"])
 # Depending on how wolfSSL was configured, the functions below may or may not be
@@ -302,6 +318,8 @@ optional_funcs = make_optional_func_list(libwolfssl_path, optional_funcs)
 source = """
     #include <wolfssl/options.h>
     #include <wolfssl/ssl.h>
+
+    int OLDTLS_ENABLED = """ + str(OLDTLS_ENABLED) + """;
 """
 ffi_source = source + openssl.source
 
@@ -316,17 +334,22 @@ ffi.set_source(
 )
 
 cdef = """
-    /**
+    /*
      * Constants
      */
     static const long SOCKET_PEER_CLOSED_E;
 
-    /**
+    /*
      * Types
      */
     typedef unsigned char byte;
     typedef unsigned int word32;
-    
+
+    extern int OLDTLS_ENABLED;
+
+    /*
+     * Opaque structs.
+     */
     typedef ... WOLFSSL_CTX;
     typedef ... WOLFSSL;
     typedef ... WOLFSSL_X509;
@@ -334,46 +357,81 @@ cdef = """
     typedef ... WOLFSSL_X509_STORE_CTX;
     typedef ... WOLFSSL_X509_NAME;
     typedef ... WOLFSSL_X509_NAME_ENTRY;
-    typedef ... WOLFSSL_ALERT_HISTORY;
     typedef ... WOLFSSL_METHOD;
     typedef ... WOLFSSL_ASN1_TIME;
     typedef ... WOLFSSL_ASN1_GENERALIZEDTIME;
     typedef ... WOLFSSL_ASN1_STRING;
     typedef ... WOLFSSL_ASN1_OBJECT;
 
+    /*
+     * Non-opaque structs, where we need access to fields.
+     */
+    typedef struct WOLFSSL_ALERT {
+        int code;
+        int level;
+    } WOLFSSL_ALERT;
+    typedef struct WOLFSSL_ALERT_HISTORY {
+        WOLFSSL_ALERT last_rx;
+        WOLFSSL_ALERT last_tx;
+    } WOLFSSL_ALERT_HISTORY;
+
     typedef int (*VerifyCallback)(int, WOLFSSL_X509_STORE_CTX*);
     typedef int pem_password_cb(char*, int, int, void*);
     typedef int (*CallbackSniRecv)(WOLFSSL*, int*, void*);
 
-    /**
+    /*
      * Memory
      */
     void  wolfSSL_Free(void*);
     void  wolfSSL_OPENSSL_free(void*);
 
-    /**
+    /*
      * Debugging
      */
     void wolfSSL_Debugging_ON();
     void wolfSSL_Debugging_OFF();
 
-    /**
+    /*
      * SSL/TLS Method functions
      */
+"""
+
+if OLDTLS_ENABLED:
+    sys.stderr.write("\nAdding OLDTLS\n")
+    cdef += """
     WOLFSSL_METHOD* wolfTLSv1_1_server_method(void);
     WOLFSSL_METHOD* wolfTLSv1_1_client_method(void);
 
+    WOLFSSL_METHOD* wolfDTLSv1_server_method(void);
+    WOLFSSL_METHOD* wolfDTLSv1_client_method(void);
+    """
+
+cdef += """
     WOLFSSL_METHOD* wolfTLSv1_2_server_method(void);
     WOLFSSL_METHOD* wolfTLSv1_2_client_method(void);
 
+    WOLFSSL_METHOD* wolfTLSv1_3_server_method(void);
+    WOLFSSL_METHOD* wolfTLSv1_3_client_method(void);
+
     WOLFSSL_METHOD* wolfSSLv23_server_method(void);
     WOLFSSL_METHOD* wolfSSLv23_client_method(void);
 
     WOLFSSL_METHOD* wolfSSLv23_method(void);
+
+    WOLFSSL_METHOD* wolfDTLSv1_2_server_method(void);
+    WOLFSSL_METHOD* wolfDTLSv1_2_client_method(void);
+
+    WOLFSSL_METHOD* wolfDTLSv1_3_server_method(void);
+    WOLFSSL_METHOD* wolfDTLSv1_3_client_method(void);
+    """
+if OLDTLS_ENABLED:
+    cdef += """
     WOLFSSL_METHOD* wolfTLSv1_1_method(void);
+    """
+cdef += """
     WOLFSSL_METHOD* wolfTLSv1_2_method(void);
 
-    /**
+    /*
      * SSL/TLS Context functions
      */
     WOLFSSL_CTX* wolfSSL_CTX_new(WOLFSSL_METHOD*);
@@ -396,7 +454,7 @@ cdef = """
             CallbackSniRecv);
     long wolfSSL_CTX_set_mode(WOLFSSL_CTX*, long);
 
-    /**
+    /*
      * SSL/TLS Session functions
      */
     void wolfSSL_Init();
@@ -422,8 +480,13 @@ cdef = """
     int           wolfSSL_set_tlsext_host_name(WOLFSSL*, const char*);
     long          wolfSSL_ctrl(WOLFSSL*, int, long, void*);
     void          wolfSSL_set_connect_state(WOLFSSL*);
+    int           wolfSSL_EnableCRL(WOLFSSL*, int);
+    int           wolfSSL_LoadCRLFile(WOLFSSL*, const char*, int);
+    void*         wolfSSL_dtls_create_peer(int, char*);
+    int           wolfSSL_dtls_free_peer(void*);
+    int           wolfSSL_dtls_set_peer(WOLFSSL*, void*, unsigned int);
 
-    /**
+    /*
      * WOLFSSL_X509 functions
      */
     char*                    wolfSSL_X509_get_subjectCN(void*);
@@ -466,7 +529,7 @@ cdef = """
                                  WOLFSSL_X509_EXTENSION*);
     WOLFSSL_X509*            wolfSSL_X509_dup(WOLFSSL_X509*);
 
-    /**
+    /*
      * ASN.1
      */
     int                wolfSSL_ASN1_STRING_length(WOLFSSL_ASN1_STRING*);
@@ -482,7 +545,7 @@ cdef = """
     int                wolfSSL_ASN1_STRING_to_UTF8(unsigned char **, 
                            WOLFSSL_ASN1_STRING*);
 
-    /**
+    /*
      * Misc.
      */
     int           wolfSSL_library_init(void);
@@ -498,7 +561,7 @@ cdef = """
 for func in optional_funcs:
     cdef += "{};".format(func.native_sig)
 
-ffi_cdef = cdef + openssl.construct_cdef(optional_funcs)
+ffi_cdef = cdef + openssl.construct_cdef(optional_funcs, OLDTLS_ENABLED)
 ffi.cdef(ffi_cdef)
 
 if __name__ == "__main__":
